@@ -56,8 +56,10 @@
 #include "ClarisWksBMParser.hxx"
 #include "ClarisWksPRParser.hxx"
 #include "ClarisWksSSParser.hxx"
+#include "CricketDrawParser.hxx"
 #include "DocMkrParser.hxx"
 #include "EDocParser.hxx"
+#include "FreeHandParser.hxx"
 #include "FullWrtParser.hxx"
 #include "GreatWksParser.hxx"
 #include "GreatWksBMParser.hxx"
@@ -76,8 +78,10 @@
 #include "MacWrtParser.hxx"
 #include "MacWrtProParser.hxx"
 #include "MarinerWrtParser.hxx"
+#include "MaxWrtParser.hxx"
 #include "MindWrtParser.hxx"
 #include "MoreParser.hxx"
+#include "MouseWrtParser.hxx"
 #include "MsWksParser.hxx"
 #include "MsWksDBParser.hxx"
 #include "MsWksDRParser.hxx"
@@ -86,6 +90,9 @@
 #include "MsWrdParser.hxx"
 #include "NisusWrtParser.hxx"
 #include "PixelPaintParser.hxx"
+#include "PowerPoint1Parser.hxx"
+#include "PowerPoint3Parser.hxx"
+#include "PowerPoint7Parser.hxx"
 #include "RagTimeParser.hxx"
 #include "RagTime5Parser.hxx"
 #include "StyleParser.hxx"
@@ -132,8 +139,8 @@ try
 
   if (!header.get())
     return MWAW_C_NONE;
-  type = (MWAWDocument::Type)header->getType();
-  kind = (MWAWDocument::Kind)header->getKind();
+  type = static_cast<MWAWDocument::Type>(header->getType());
+  kind = static_cast<MWAWDocument::Kind>(header->getKind());
   Confidence confidence = MWAW_C_NONE;
 
   switch (type) {
@@ -143,8 +150,10 @@ try
   case MWAW_T_CLARISDRAW:
   case MWAW_T_CLARISRESOLVE:
   case MWAW_T_CLARISWORKS:
+  case MWAW_T_CRICKETDRAW:
   case MWAW_T_DOCMAKER:
   case MWAW_T_EDOC:
+  case MWAW_T_FREEHAND:
   case MWAW_T_FULLPAINT:
   case MWAW_T_FULLWRITE:
   case MWAW_T_GREATWORKS:
@@ -159,12 +168,15 @@ try
   case MWAW_T_MACWRITE:
   case MWAW_T_MACWRITEPRO:
   case MWAW_T_MARINERWRITE:
+  case MWAW_T_MAXWRITE:
   case MWAW_T_MINDWRITE:
   case MWAW_T_MORE:
+  case MWAW_T_MOUSEWRITE:
   case MWAW_T_MICROSOFTWORD:
   case MWAW_T_MICROSOFTWORKS:
   case MWAW_T_NISUSWRITER:
   case MWAW_T_PIXELPAINT:
+  case MWAW_T_POWERPOINT:
   case MWAW_T_RAGTIME:
   case MWAW_T_STYLE:
   case MWAW_T_SUPERPAINT:
@@ -196,14 +208,11 @@ try
   case MWAW_T_XPRESS:
   case MWAW_T_4DIMENSION:
 
-  case MWAW_T_RESERVED4:
-  case MWAW_T_RESERVED5:
-  case MWAW_T_RESERVED6:
-  case MWAW_T_RESERVED7:
-  case MWAW_T_RESERVED8:
   case MWAW_T_RESERVED9:
   case MWAW_T_UNKNOWN:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
 
@@ -447,10 +456,13 @@ try
   size_t numHeaders = listHeaders.size();
   if (numHeaders==0) return 0L;
 
-  for (size_t i = 0; i < numHeaders; i++) {
-    if (!MWAWDocumentInternal::checkBasicMacHeader(ip, rsrcParser, listHeaders[i], strict))
-      continue;
-    return new MWAWHeader(listHeaders[i]);
+  for (int step=0; step<2; ++step) {
+    for (size_t i = 0; i < numHeaders; i++) {
+      if (!MWAWDocumentInternal::checkBasicMacHeader(ip, rsrcParser, listHeaders[i], step==0))
+        continue;
+      return new MWAWHeader(listHeaders[i]);
+    }
+    if (strict) break;
   }
   return 0L;
 }
@@ -499,9 +511,15 @@ shared_ptr<MWAWGraphicParser> getGraphicParserFromHeader(MWAWInputStreamPtr &inp
       if (header->getKind()==MWAWDocument::MWAW_K_PAINT)
         parser.reset(new ClarisWksBMParser(input, rsrcParser, header));
       break;
+    case MWAWDocument::MWAW_T_CRICKETDRAW:
+      parser.reset(new CricketDrawParser(input, rsrcParser, header));
+      break;
     case MWAWDocument::MWAW_T_FULLPAINT: // same format as MacPaint, so ok...
       parser.reset(new MacPaintParser(input, rsrcParser, header));
       break;
+    case MWAWDocument::MWAW_T_FREEHAND:
+      parser.reset(new FreeHandParser(input, rsrcParser, header));
+      break;
     case MWAWDocument::MWAW_T_GREATWORKS:
       if (header->getKind()==MWAWDocument::MWAW_K_PAINT)
         parser.reset(new GreatWksBMParser(input, rsrcParser, header));
@@ -554,14 +572,17 @@ shared_ptr<MWAWGraphicParser> getGraphicParserFromHeader(MWAWInputStreamPtr &inp
     case MWAWDocument::MWAW_T_MACWRITE:
     case MWAWDocument::MWAW_T_MACWRITEPRO:
     case MWAWDocument::MWAW_T_MARINERWRITE:
+    case MWAWDocument::MWAW_T_MAXWRITE:
     case MWAWDocument::MWAW_T_MINDWRITE:
     case MWAWDocument::MWAW_T_MICROSOFTFILE:
     case MWAWDocument::MWAW_T_MICROSOFTMULTIPLAN:
     case MWAWDocument::MWAW_T_MICROSOFTWORD:
     case MWAWDocument::MWAW_T_MORE:
+    case MWAWDocument::MWAW_T_MOUSEWRITE:
     case MWAWDocument::MWAW_T_NISUSWRITER:
     case MWAWDocument::MWAW_T_OVERVUE:
     case MWAWDocument::MWAW_T_PAGEMAKER:
+    case MWAWDocument::MWAW_T_POWERPOINT:
     case MWAWDocument::MWAW_T_RAGTIME:
     case MWAWDocument::MWAW_T_READYSETGO:
     case MWAWDocument::MWAW_T_STYLE:
@@ -576,14 +597,11 @@ shared_ptr<MWAWGraphicParser> getGraphicParserFromHeader(MWAWInputStreamPtr &inp
     case MWAWDocument::MWAW_T_ZWRITE:
     case MWAWDocument::MWAW_T_4DIMENSION:
 
-    case MWAWDocument::MWAW_T_RESERVED4:
-    case MWAWDocument::MWAW_T_RESERVED5:
-    case MWAWDocument::MWAW_T_RESERVED6:
-    case MWAWDocument::MWAW_T_RESERVED7:
-    case MWAWDocument::MWAW_T_RESERVED8:
     case MWAWDocument::MWAW_T_RESERVED9:
     case MWAWDocument::MWAW_T_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       break;
     }
   }
@@ -605,17 +623,28 @@ shared_ptr<MWAWPresentationParser> getPresentationParserFromHeader(MWAWInputStre
     case MWAWDocument::MWAW_T_CLARISWORKS:
       parser.reset(new ClarisWksPRParser(input, rsrcParser, header));
       break;
+    case MWAWDocument::MWAW_T_POWERPOINT:
+      if (header->getMajorVersion()<=2)
+        parser.reset(new PowerPoint1Parser(input, rsrcParser, header));
+      else if (header->getMajorVersion()<=4)
+        parser.reset(new PowerPoint3Parser(input, rsrcParser, header));
+      else
+        parser.reset(new PowerPoint7Parser(input, rsrcParser, header));
+      break;
+
     case MWAWDocument::MWAW_T_ACTA:
     case MWAWDocument::MWAW_T_ADOBEILLUSTRATOR:
     case MWAWDocument::MWAW_T_APPLEPICT:
     case MWAWDocument::MWAW_T_BEAGLEWORKS:
     case MWAWDocument::MWAW_T_CLARISDRAW:
     case MWAWDocument::MWAW_T_CLARISRESOLVE:
+    case MWAWDocument::MWAW_T_CRICKETDRAW:
     case MWAWDocument::MWAW_T_DBASE:
     case MWAWDocument::MWAW_T_DOCMAKER:
     case MWAWDocument::MWAW_T_EDOC:
     case MWAWDocument::MWAW_T_FAMILYTREEMAKER:
     case MWAWDocument::MWAW_T_FILEMAKER:
+    case MWAWDocument::MWAW_T_FREEHAND:
     case MWAWDocument::MWAW_T_FOXBASE:
     case MWAWDocument::MWAW_T_FRAMEMAKER:
     case MWAWDocument::MWAW_T_FULLIMPACT:
@@ -635,12 +664,14 @@ shared_ptr<MWAWPresentationParser> getPresentationParserFromHeader(MWAWInputStre
     case MWAWDocument::MWAW_T_MACWRITE:
     case MWAWDocument::MWAW_T_MACWRITEPRO:
     case MWAWDocument::MWAW_T_MARINERWRITE:
+    case MWAWDocument::MWAW_T_MAXWRITE:
     case MWAWDocument::MWAW_T_MINDWRITE:
     case MWAWDocument::MWAW_T_MICROSOFTFILE:
     case MWAWDocument::MWAW_T_MICROSOFTMULTIPLAN:
     case MWAWDocument::MWAW_T_MICROSOFTWORD:
     case MWAWDocument::MWAW_T_MICROSOFTWORKS:
     case MWAWDocument::MWAW_T_MORE:
+    case MWAWDocument::MWAW_T_MOUSEWRITE:
     case MWAWDocument::MWAW_T_NISUSWRITER:
     case MWAWDocument::MWAW_T_OVERVUE:
     case MWAWDocument::MWAW_T_PAGEMAKER:
@@ -660,14 +691,11 @@ shared_ptr<MWAWPresentationParser> getPresentationParserFromHeader(MWAWInputStre
     case MWAWDocument::MWAW_T_ZWRITE:
     case MWAWDocument::MWAW_T_4DIMENSION:
 
-    case MWAWDocument::MWAW_T_RESERVED4:
-    case MWAWDocument::MWAW_T_RESERVED5:
-    case MWAWDocument::MWAW_T_RESERVED6:
-    case MWAWDocument::MWAW_T_RESERVED7:
-    case MWAWDocument::MWAW_T_RESERVED8:
     case MWAWDocument::MWAW_T_RESERVED9:
     case MWAWDocument::MWAW_T_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       break;
     }
   }
@@ -725,6 +753,7 @@ shared_ptr<MWAWSpreadsheetParser> getSpreadsheetParserFromHeader(MWAWInputStream
     case MWAWDocument::MWAW_T_ADOBEILLUSTRATOR:
     case MWAWDocument::MWAW_T_APPLEPICT:
     case MWAWDocument::MWAW_T_CLARISDRAW:
+    case MWAWDocument::MWAW_T_CRICKETDRAW:
     case MWAWDocument::MWAW_T_DBASE:
     case MWAWDocument::MWAW_T_DOCMAKER:
     case MWAWDocument::MWAW_T_EDOC:
@@ -732,6 +761,7 @@ shared_ptr<MWAWSpreadsheetParser> getSpreadsheetParserFromHeader(MWAWInputStream
     case MWAWDocument::MWAW_T_FILEMAKER:
     case MWAWDocument::MWAW_T_FOXBASE:
     case MWAWDocument::MWAW_T_FRAMEMAKER:
+    case MWAWDocument::MWAW_T_FREEHAND:
     case MWAWDocument::MWAW_T_FULLPAINT:
     case MWAWDocument::MWAW_T_FULLWRITE:
     case MWAWDocument::MWAW_T_HANMACWORDJ:
@@ -746,13 +776,16 @@ shared_ptr<MWAWSpreadsheetParser> getSpreadsheetParserFromHeader(MWAWInputStream
     case MWAWDocument::MWAW_T_MACWRITE:
     case MWAWDocument::MWAW_T_MACWRITEPRO:
     case MWAWDocument::MWAW_T_MARINERWRITE:
+    case MWAWDocument::MWAW_T_MAXWRITE:
     case MWAWDocument::MWAW_T_MICROSOFTFILE:
     case MWAWDocument::MWAW_T_MICROSOFTWORD:
     case MWAWDocument::MWAW_T_MINDWRITE:
     case MWAWDocument::MWAW_T_MORE:
+    case MWAWDocument::MWAW_T_MOUSEWRITE:
     case MWAWDocument::MWAW_T_NISUSWRITER:
     case MWAWDocument::MWAW_T_OVERVUE:
     case MWAWDocument::MWAW_T_PAGEMAKER:
+    case MWAWDocument::MWAW_T_POWERPOINT:
     case MWAWDocument::MWAW_T_PIXELPAINT:
     case MWAWDocument::MWAW_T_RAGTIME:
     case MWAWDocument::MWAW_T_READYSETGO:
@@ -767,14 +800,11 @@ shared_ptr<MWAWSpreadsheetParser> getSpreadsheetParserFromHeader(MWAWInputStream
     case MWAWDocument::MWAW_T_ZWRITE:
     case MWAWDocument::MWAW_T_4DIMENSION:
 
-    case MWAWDocument::MWAW_T_RESERVED4:
-    case MWAWDocument::MWAW_T_RESERVED5:
-    case MWAWDocument::MWAW_T_RESERVED6:
-    case MWAWDocument::MWAW_T_RESERVED7:
-    case MWAWDocument::MWAW_T_RESERVED8:
     case MWAWDocument::MWAW_T_RESERVED9:
     case MWAWDocument::MWAW_T_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       break;
     }
   }
@@ -837,6 +867,9 @@ shared_ptr<MWAWTextParser> getTextParserFromHeader(MWAWInputStreamPtr &input, MW
     case MWAWDocument::MWAW_T_MARINERWRITE:
       parser.reset(new MarinerWrtParser(input, rsrcParser, header));
       break;
+    case MWAWDocument::MWAW_T_MAXWRITE:
+      parser.reset(new MaxWrtParser(input, rsrcParser, header));
+      break;
     case MWAWDocument::MWAW_T_MICROSOFTWORD:
       if (header->getMajorVersion()==1)
         parser.reset(new MsWrd1Parser(input, rsrcParser, header));
@@ -852,6 +885,9 @@ shared_ptr<MWAWTextParser> getTextParserFromHeader(MWAWInputStreamPtr &input, MW
     case MWAWDocument::MWAW_T_MORE:
       parser.reset(new MoreParser(input, rsrcParser, header));
       break;
+    case MWAWDocument::MWAW_T_MOUSEWRITE:
+      parser.reset(new MouseWrtParser(input, rsrcParser, header));
+      break;
     case MWAWDocument::MWAW_T_NISUSWRITER:
       parser.reset(new NisusWrtParser(input, rsrcParser, header));
       break;
@@ -882,10 +918,12 @@ shared_ptr<MWAWTextParser> getTextParserFromHeader(MWAWInputStreamPtr &input, MW
     case MWAWDocument::MWAW_T_APPLEPICT:
     case MWAWDocument::MWAW_T_CLARISDRAW:
     case MWAWDocument::MWAW_T_CLARISRESOLVE:
+    case MWAWDocument::MWAW_T_CRICKETDRAW:
     case MWAWDocument::MWAW_T_DBASE:
     case MWAWDocument::MWAW_T_FAMILYTREEMAKER:
     case MWAWDocument::MWAW_T_FILEMAKER:
     case MWAWDocument::MWAW_T_FOXBASE:
+    case MWAWDocument::MWAW_T_FREEHAND:
     case MWAWDocument::MWAW_T_FULLIMPACT:
     case MWAWDocument::MWAW_T_FULLPAINT:
     case MWAWDocument::MWAW_T_FRAMEMAKER:
@@ -900,6 +938,7 @@ shared_ptr<MWAWTextParser> getTextParserFromHeader(MWAWInputStreamPtr &input, MW
     case MWAWDocument::MWAW_T_OVERVUE:
     case MWAWDocument::MWAW_T_PAGEMAKER:
     case MWAWDocument::MWAW_T_PIXELPAINT:
+    case MWAWDocument::MWAW_T_POWERPOINT:
     case MWAWDocument::MWAW_T_READYSETGO:
     case MWAWDocument::MWAW_T_SUPERPAINT:
     case MWAWDocument::MWAW_T_SYMPOSIUM:
@@ -908,14 +947,11 @@ shared_ptr<MWAWTextParser> getTextParserFromHeader(MWAWInputStreamPtr &input, MW
     case MWAWDocument::MWAW_T_XPRESS:
     case MWAWDocument::MWAW_T_4DIMENSION:
 
-    case MWAWDocument::MWAW_T_RESERVED4:
-    case MWAWDocument::MWAW_T_RESERVED5:
-    case MWAWDocument::MWAW_T_RESERVED6:
-    case MWAWDocument::MWAW_T_RESERVED7:
-    case MWAWDocument::MWAW_T_RESERVED8:
     case MWAWDocument::MWAW_T_RESERVED9:
     case MWAWDocument::MWAW_T_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       break;
     }
   }
