@@ -63,6 +63,8 @@ class Cell : public MWAWCell
 public:
   /// constructor
   Cell() : m_content(), m_noteId(0) { }
+  //! destructor
+  virtual ~Cell();
   //! returns true if the cell do contain any content
   bool isEmpty() const
   {
@@ -121,19 +123,23 @@ public:
   std::string m_name;
 protected:
   /** returns the last Right Bottom cell position */
-  Vec2i getRightBottomPosition() const
+  MWAWVec2i getRightBottomPosition() const
   {
     int maxX = 0, maxY = 0;
     size_t numCell = m_cells.size();
     for (size_t i = 0; i < numCell; i++) {
-      Vec2i const &p = m_cells[i].position();
+      MWAWVec2i const &p = m_cells[i].position();
       if (p[0] > maxX) maxX = p[0];
       if (p[1] > maxY) maxY = p[1];
     }
-    return Vec2i(maxX, maxY);
+    return MWAWVec2i(maxX, maxY);
   }
 };
 
+Cell::~Cell()
+{
+}
+
 ////////////////////////////////////////
 //! Internal: the state of a MsWksSSParser
 struct State {
@@ -180,13 +186,15 @@ protected:
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*type*/)
 {
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("MsWksSSParser::SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("MsWksSSParserInternal::SubDocument::parse: no listener\n"));
+    return;
+  }
+  MsWksSSParser *parser = dynamic_cast<MsWksSSParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("MsWksSSParserInternal::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
-
   long pos = m_input->tell();
-  MsWksSSParser *parser = static_cast<MsWksSSParser *>(m_parser);
   parser->sendNote(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
@@ -245,7 +253,7 @@ void MsWksSSParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 #ifdef DEBUG
     MWAWInputStreamPtr &input= getInput();
     if (input->isStructured()) {
-      shared_ptr<MWAWOLEParser> oleParser(new MWAWOLEParser("MN0"));
+      shared_ptr<MWAWOLEParser> oleParser(new MWAWOLEParser("MN0", getParserState()->m_fontConverter, 3));
       oleParser->parse(input);
     }
 #endif
@@ -286,7 +294,7 @@ void MsWksSSParser::sendNote(int noteId)
   while (1) {
     long pos=input->tell();
     if (input->isEnd() || pos>=entry.end()) return;
-    int c = (int) input->readULong(1);
+    int c = static_cast<int>(input->readULong(1));
     switch (c) {
     case 0x9:
       listener->insertTab();
@@ -320,10 +328,10 @@ void MsWksSSParser::sendNote(int noteId)
         }
       }
       else if (c <= 0x1f) {
-        MWAW_DEBUG_MSG(("MsWksSSParser::sendNote: find char=%x\n",int(c)));
+        MWAW_DEBUG_MSG(("MsWksSSParser::sendNote: find char=%x\n", static_cast<unsigned int>(c)));
       }
       else
-        listener->insertCharacter((unsigned char)c, input, entry.end());
+        listener->insertCharacter(static_cast<unsigned char>(c), input, entry.end());
       break;
     }
 
@@ -349,8 +357,8 @@ void MsWksSSParser::createDocument(librevenge::RVNGSpreadsheetInterface *documen
   listen->startDocument();
   // time to send page information the graph parser and the text parser
   m_document->getGraphParser()->setPageLeftTop
-  (Vec2f(72.f*float(getPageSpan().getMarginLeft()),
-         72.f*float(getPageSpan().getMarginTop())+m_document->getHeaderFooterHeight(true)));
+  (MWAWVec2f(72.f*float(getPageSpan().getMarginLeft()),
+             72.f*float(getPageSpan().getMarginTop())+m_document->getHeaderFooterHeight(true)));
 }
 
 
@@ -405,7 +413,7 @@ bool MsWksSSParser::createZones()
     MWAW_DEBUG_MSG(("MsWksSSParser::createZones: find some unexpected data\n"));
     pos = input->tell();
     MsWksDocument::Zone unknown;
-    if (!m_document->readZone(unknown)) {
+    if (!m_document->readZone(unknown) || input->tell() <= pos) {
       ascFile.addPos(pos);
       ascFile.addNote("Entries(End)");
       ascFile.addPos(pos+100);
@@ -448,32 +456,32 @@ bool MsWksSSParser::readSSheetZone()
 
   int N[2]; // max col, max row
   for (int i = 0; i < 2; i++)
-    N[i] = (int) input->readLong(2);
+    N[i] = static_cast<int>(input->readLong(2));
   f << "maxCol=" << N[0] << ","  << "maxRow=" << N[1] << ",";
 
   f << "unk0=" << std::hex << input->readULong(4) << std::dec << ","; // big number
 
   int zoneSize[3];
-  zoneSize[0] = (int) input->readLong(2);
+  zoneSize[0] = static_cast<int>(input->readLong(2));
   if (zoneSize[0]) f << "zone[SheetF]=" << std::hex << zoneSize[0] << std::dec << ",";
-  int numPageBreak = (int) input->readLong(2);
+  int numPageBreak = static_cast<int>(input->readLong(2));
   if (numPageBreak) f << "nPgBreak=" << numPageBreak << ",";
-  int numColBreak = (int) input->readLong(2);
+  int numColBreak = static_cast<int>(input->readLong(2));
   if (numColBreak) f << "nColBreak=" << numColBreak << ",";
-  zoneSize[1]= (int) input->readLong(2);
+  zoneSize[1]= static_cast<int>(input->readLong(2));
   if (zoneSize[1]) f << "zone[DocInfo]=" << zoneSize[1] << ",";
   f << "unk1=" << std::hex << input->readULong(4) << std::dec << ","; // big number
   f << std::dec;
 
   int val;
   for (int i = 0; i < 2; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << "w" << i << "=" << val << ",";
   }
   f << "v2=["; // [_,_,_,_,_,_,] or [_,1,3,_,1,3,] or [_,5,5,_,4,2,]
   for (int i = 0; i < 6; i++) {
-    val = (int) input->readLong(1);
+    val = static_cast<int>(input->readLong(1));
     if (val)
       f << val << ",";
     else
@@ -482,7 +490,7 @@ bool MsWksSSParser::readSSheetZone()
   f << "]";
   f << ", v3=["; // [_,_,_,_,] or [_,_,_,1,] or [_,_,_,11,]
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << val << ",";
     else
@@ -492,7 +500,7 @@ bool MsWksSSParser::readSSheetZone()
 
   f << ", v4=[" << std::hex;
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val)
       f << val << ",";
     else
@@ -512,12 +520,12 @@ bool MsWksSSParser::readSSheetZone()
   int prevW = -1;
   bool ok = true;
   for (int i = 0; i < 257; i++) {
-    int col = (int) input->readULong(2);
+    int col = static_cast<int>(input->readULong(2));
     if ((col & 0xFFF) != i+1) {
       ok = false;
       break;
     }
-    int w = (int) input->readLong(2);
+    int w = static_cast<int>(input->readLong(2));
     if (w > 0) {
       if (prevW > w && i < 200) {
         ok = false;
@@ -569,8 +577,8 @@ bool MsWksSSParser::readSSheetZone()
   for (int i=0; i < 8; ++i) {
     // the chart names in v2, can we have more than 8 charts ?
     long cPos=input->tell();
-    val = (int) input->readLong(2);
-    int sSz=(int) input->readULong(1);
+    val = static_cast<int>(input->readLong(2));
+    int sSz=static_cast<int>(input->readULong(1));
     if (!sSz || sSz>32-7) {
       input->seek(cPos, librevenge::RVNG_SEEK_SET);
       break;
@@ -580,7 +588,7 @@ bool MsWksSSParser::readSSheetZone()
     if (val) f << "unkn=" << val << ",";
     std::string name("");
     for (int c=0; c<sSz; ++c)
-      name += (char) input->readULong(1);
+      name += char(input->readULong(1));
     f << name << ",";
     if ((sSz%2)==0)   input->seek(1, librevenge::RVNG_SEEK_CUR);
     f << "id=" << std::hex << input->readULong(4) << std::dec << "],";
@@ -588,7 +596,7 @@ bool MsWksSSParser::readSSheetZone()
   }
   int numRemains=int(endPos-input->tell())/2;
   for (int i = 0; i < numRemains; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascFile.addNote(f.str().c_str());
@@ -606,7 +614,7 @@ bool MsWksSSParser::readSSheetZone()
 
   f << "SSheetD:";
   for (int i = 0; i < 5; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (i==3) {
       if (val==44) continue;
       f << "unkn=" << val << ",";
@@ -623,9 +631,9 @@ bool MsWksSSParser::readSSheetZone()
   for (int i = 0; i < 2; i++) {
     f << ", g" << i << "="  << std::hex << input->readLong(2) << std::dec;
   }
-  int fId = (int) input->readLong(2);
-  int fSize = (int) input->readLong(2);
-  sheet.m_font = MWAWFont(fId, (float) fSize);
+  int fId = static_cast<int>(input->readLong(2));
+  int fSize = static_cast<int>(input->readLong(2));
+  sheet.m_font = MWAWFont(fId, float(fSize));
   f << ", [" <<  sheet.m_font.getDebugString(getParserState()->m_fontConverter) << "]";
   // , [0,{33|255},] ?
   f << ", unk=[" << std::dec;
@@ -634,20 +642,19 @@ bool MsWksSSParser::readSSheetZone()
   f << "]";
   int numRemain=int(sheetDebPos+m_document->getLengthOfFileHeader3()-input->tell())/2;
   for (int i = 0; i < numRemain; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   ascFile.addNote(f.str().c_str());
   input->seek(sheetDebPos+m_document->getLengthOfFileHeader3(), librevenge::RVNG_SEEK_SET);
 
   // part E
-  Vec2i cellPos(0,0);
+  MWAWVec2i cellPos(0,0);
   while (1) {
     if (cellPos[1] == N[1]) break;
 
     long posRow = input->tell();
     unsigned long length = input->readULong(4);
-    int fl = 0;
     if (length & 0xF0000000L) {
       if ((length >> 28) != 8) {
         input->seek(posRow, librevenge::RVNG_SEEK_SET);
@@ -661,7 +668,7 @@ bool MsWksSSParser::readSSheetZone()
       ascFile.addNote(f.str().c_str());
       continue;
     }
-    long endRow = posRow+(long)length+4;
+    long endRow = posRow+long(length)+4;
 
     if (!input->checkPosition(endRow)) {
       MWAW_DEBUG_MSG(("MsWksSSParser::readSSheetZone:Can not read part E(row)\n"));
@@ -672,13 +679,12 @@ bool MsWksSSParser::readSSheetZone()
     f.str("");
     cellPos[0]=0;
     f << "SSheetE(Row" << std::dec << cellPos[1]+1 << "): ";
-    if (fl) f << std::hex << "fl=" << fl << ",";
     while (input->tell() < endRow-1) {
       pos = input->tell();
-      int sz = (int) input->readULong(1);
+      int sz = static_cast<int>(input->readULong(1));
 
       if (sz == 0xFE) { // skip next cell
-        int numC = (int) input->readULong(1);
+        int numC = static_cast<int>(input->readULong(1));
         f2.str("");
         f2 << "SSheetE:skipC=" << numC;
         cellPos[0]+=numC;
@@ -710,7 +716,7 @@ bool MsWksSSParser::readSSheetZone()
       return false;
     }
     ascFile.addDelimiter(endRow-1,'|');
-    val = (int) input->readULong(1);
+    val = static_cast<int>(input->readULong(1));
     if (val == 0) f << "*";
     else if (val != 0xFF)
       f << "##end=" << std::hex << val << std::dec;
@@ -738,10 +744,10 @@ bool MsWksSSParser::readSSheetZone()
   f.str("");
   f << "SSheetF[B]:" << std::dec;
   for (int i = 0; i < 4; i++) {
-    val = (int) input->readLong(2);
+    val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  val = (int) input->readULong(2);
+  val = static_cast<int>(input->readULong(2));
   if (val) f << std::hex << "fl=" << val << "," << std::dec;
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
@@ -751,10 +757,10 @@ bool MsWksSSParser::readSSheetZone()
     f.str("");
     f << "SSheetG:";
     if (numPageBreak > 0) {
-      sheet.m_listPageBreaks.resize((size_t) numPageBreak);
+      sheet.m_listPageBreaks.resize(size_t(numPageBreak));
       f << "pgBreak=[";
-      for (size_t pgBreak = 0; pgBreak < (size_t) numPageBreak; pgBreak++) {
-        val = (int) input->readULong(2);
+      for (size_t pgBreak = 0; pgBreak < size_t(numPageBreak); pgBreak++) {
+        val = static_cast<int>(input->readULong(2));
         sheet.m_listPageBreaks[pgBreak] = val;
         f << val << ",";
       }
@@ -805,7 +811,7 @@ bool MsWksSSParser::readSSheetZone()
   int numNote = 0;
   while (1) {
     pos = input->tell();
-    int idNote = (int) input->readLong(2);
+    int idNote = static_cast<int>(input->readLong(2));
     if (idNote == 0) break;
     if (idNote != ++numNote) {
       input->seek(pos, librevenge::RVNG_SEEK_CUR);
@@ -813,13 +819,13 @@ bool MsWksSSParser::readSSheetZone()
       return false;
     }
     // a pascal string
-    int pSz = (int) input->readLong(2);
+    int pSz = static_cast<int>(input->readLong(2));
     MWAWEntry note;
     note.setBegin(input->tell());
     note.setLength(pSz);
     std::string str="";
     for (int s = 0; s < pSz; s++)
-      str += (char) input->readLong(1);
+      str += char(input->readLong(1));
     if (input->tell() != pos+4+pSz) {
       input->seek(pos, librevenge::RVNG_SEEK_CUR);
       MWAW_DEBUG_MSG(("MsWksSSParser::readSSheetZone:Pb when reading a note\n"));
@@ -836,7 +842,7 @@ bool MsWksSSParser::readSSheetZone()
   return true;
 }
 
-bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal::Cell &cell)
+bool MsWksSSParser::readCell(int sz, MWAWVec2i const &cellPos, MsWksSSParserInternal::Cell &cell)
 {
   int const vers=version();
   cell = MsWksSSParserInternal::Cell();
@@ -856,37 +862,43 @@ bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal
   input->seek(endPos, librevenge::RVNG_SEEK_SET);
   int fl[5];
   for (int i  = 0; i < 5; i++)
-    fl[i] = (int) input->readULong(1);
+    fl[i] = static_cast<int>(input->readULong(1));
   MWAWCell::Format format;
   MWAWCellContent &content=cell.m_content;
-  int style = 0;
-  int subformat = 0;
+  int style = 0, align = 0, subformat = 0;
   if (vers >= 3) {
     style = (fl[4]>>1) & 0x1f;
-    switch (fl[4] >> 6) {
-    case 0:
-      cell.setHAlignement(MWAWCell::HALIGN_LEFT);
-      break;
-    case 1:
-      cell.setHAlignement(MWAWCell::HALIGN_CENTER);
-      break;
-    case 2:
-      cell.setHAlignement(MWAWCell::HALIGN_RIGHT);
-      break;
-    default:
-    case 3:
-      break; // default
-    }
+    align = (fl[4]>>6);
     fl[4] &= 1;
     subformat = fl[2] >> 4;
     cell.setBorders(fl[2] & 0xF, MWAWBorder()); // checkme
     fl[2] = 0;
   }
+  else {
+    style = fl[4] & 0x18;
+    align = (fl[4]>>5)&3;
+    subformat= (fl[4]&7);
+    fl[4] &= 0x80;
+  }
+  switch (align) {
+  case 0:
+    cell.setHAlignment(MWAWCell::HALIGN_LEFT);
+    break;
+  case 1:
+    cell.setHAlignment(MWAWCell::HALIGN_CENTER);
+    break;
+  case 2:
+    cell.setHAlignment(MWAWCell::HALIGN_RIGHT);
+    break;
+  default:
+  case 3:
+    break; // default
+  }
   cell.m_noteId = fl[1] & 0xf;
   fl[1] >>= 4;
   int type = fl[3] >> 4;
   format.m_digits=(fl[3] & 0xf);
-  fl[3] = 0;
+  fl[3] &= 1;
 
   MWAWFont font=m_state->m_spreadsheet.m_font;
   MWAWColor col;
@@ -906,131 +918,139 @@ bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal
   cell.setFont(font);
 
   content.m_contentType=MWAWCellContent::C_NONE;
-  if (type & 2) {
+  if (type & 2) { // checkme: is this also ok for v2?
     cell.setProtected(true);
     type &= 0xFD;
   }
-  switch (type) {
+  switch (type>>2) {
   case 0:
     content.m_contentType=MWAWCellContent::C_TEXT;
     format.m_format=MWAWCell::F_TEXT;
     break;
   case 1:
-    if (subformat < 4)
-      format.m_format=MWAWCell::F_TIME;
-    else
-      format.m_format=MWAWCell::F_DATE;
-    content.m_contentType=MWAWCellContent::C_TEXT;
-    break;
-  case 4:
     format.m_format=MWAWCell::F_NUMBER;
     content.m_contentType=MWAWCellContent::C_NUMBER;
     break;
-  case 5:
-    if (subformat < 4)
-      format.m_format=MWAWCell::F_TIME;
-    else
-      format.m_format=MWAWCell::F_DATE;
-    content.m_contentType=MWAWCellContent::C_NUMBER;
-    break;
-  case 12:
+  case 3:
     f << "type" << type << ",";
-  case 8: //number general
+  case 2: //number general
     format.m_format=MWAWCell::F_NUMBER;
     content.m_contentType=MWAWCellContent::C_FORMULA;
     break;
-  case 13: // date or time
-    f << "type" << type << ",";
-  case 9: // date or time
-    if (subformat < 4)
-      format.m_format=MWAWCell::F_TIME;
-    else
-      format.m_format=MWAWCell::F_DATE;
-    content.m_contentType=MWAWCellContent::C_FORMULA;
-    break;
   default:
-    MWAW_DEBUG_MSG(("MsWksSSParser::readCell: unknown type:%d for a cell\n", type));
-    f << "typ[##unk"<<type << "],";
-    content.m_contentType=MWAWCellContent::C_UNKNOWN;
     break;
   }
-  switch (format.m_format) {
-  case  MWAWCell::F_NUMBER:
-    format.m_numberFormat=MWAWCell::F_NUMBER_GENERIC;
+  if (vers <= 2 && format.m_format==MWAWCell::F_NUMBER) {
     switch (subformat) {
     case 0:
-      break;
-    case 1:
       format.m_numberFormat=MWAWCell::F_NUMBER_DECIMAL;
       break;
-    case 2:
+    case 1:
       format.m_numberFormat=MWAWCell::F_NUMBER_CURRENCY;
       break;
-    case 3:
-      format.m_thousandHasSeparator=true;
+    case 2:
+      format.m_numberFormat=MWAWCell::F_NUMBER_PERCENT;
       break;
     case 4:
-      format.m_numberFormat=MWAWCell::F_NUMBER_CURRENCY;
-      format.m_thousandHasSeparator=true;
-      break;
-    case 5:
       format.m_numberFormat=MWAWCell::F_NUMBER_SCIENTIFIC;
       break;
-    case 6:
-      format.m_numberFormat=MWAWCell::F_NUMBER_PERCENT;
-      break;
-    case 7:
-      format.m_numberFormat=MWAWCell::F_NUMBER_PERCENT;
-      format.m_thousandHasSeparator=true;
-      break;
-    default:
-      f << ",subform==##unkn" << subformat;
-      break;
-    }
-    break;
-  case MWAWCell::F_TIME:
-    if (subformat >= 0 && subformat < 4) {
-      static char const *(wh[])= {"%I:%M:%S %p", "%I:%M %p", "%H:%M:%S", "%H:%M"};
-      format.m_DTFormat=wh[subformat];
-    }
-    else
-      f << ",subform==##unkn" << subformat;
-    break;
-  case MWAWCell::F_DATE:
-    switch (subformat) {
-    case 4:
     case 5:
     case 6:
+      format.m_format=MWAWCell::F_DATE;
+      break;
     case 7:
-    case 8: {
-      static char const *(wh[])= {"%m/%d/%y", "%b %d, %Y", "%b, %d", "%b, %Y", "%a, %d %b, %Y" };
-      format.m_DTFormat=wh[subformat-4];
+      format.m_format=MWAWCell::F_TIME;
+      break;
+    case 3: // generic
+    default:
+      format.m_numberFormat=MWAWCell::F_NUMBER_GENERIC;
       break;
     }
-    case 10:
-    case 11:
-      format.m_DTFormat="%B %d %Y";
+  }
+  else if (vers>2) {
+    if (type&1) {
+      format.m_format= (subformat < 4) ? MWAWCell::F_TIME : MWAWCell::F_DATE;
+      fl[3] = 0;
+    }
+    switch (format.m_format) {
+    case  MWAWCell::F_NUMBER:
+      format.m_numberFormat=MWAWCell::F_NUMBER_GENERIC;
+      switch (subformat) {
+      case 0:
+        break;
+      case 1:
+        format.m_numberFormat=MWAWCell::F_NUMBER_DECIMAL;
+        break;
+      case 2:
+        format.m_numberFormat=MWAWCell::F_NUMBER_CURRENCY;
+        break;
+      case 3:
+        format.m_thousandHasSeparator=true;
+        break;
+      case 4:
+        format.m_numberFormat=MWAWCell::F_NUMBER_CURRENCY;
+        format.m_thousandHasSeparator=true;
+        break;
+      case 5:
+        format.m_numberFormat=MWAWCell::F_NUMBER_SCIENTIFIC;
+        break;
+      case 6:
+        format.m_numberFormat=MWAWCell::F_NUMBER_PERCENT;
+        break;
+      case 7:
+        format.m_numberFormat=MWAWCell::F_NUMBER_PERCENT;
+        format.m_thousandHasSeparator=true;
+        break;
+      default:
+        f << ",subform==##unkn" << subformat;
+        break;
+      }
       break;
-    case 12:
-    case 13:
-      format.m_DTFormat="%A, %B %d, %Y";
+    case MWAWCell::F_TIME:
+      if (subformat >= 0 && subformat < 4) {
+        static char const *(wh[])= {"%I:%M:%S %p", "%I:%M %p", "%H:%M:%S", "%H:%M"};
+        format.m_DTFormat=wh[subformat];
+      }
+      else
+        f << ",subform==##unkn" << subformat;
       break;
+    case MWAWCell::F_DATE:
+      switch (subformat) {
+      case 4:
+      case 5:
+      case 6:
+      case 7:
+      case 8: {
+        static char const *(wh[])= {"%m/%d/%y", "%b %d, %Y", "%b, %d", "%b, %Y", "%a, %d %b, %Y" };
+        format.m_DTFormat=wh[subformat-4];
+        break;
+      }
+      case 10:
+      case 11:
+        format.m_DTFormat="%B %d %Y";
+        break;
+      case 12:
+      case 13:
+        format.m_DTFormat="%A, %B %d, %Y";
+        break;
+      default:
+        f << ",subform==##unkn" << subformat;
+        break;
+      }
+      break;
+    case MWAWCell::F_TEXT:
+      break;
+    case MWAWCell::F_BOOLEAN:
+    case MWAWCell::F_UNKNOWN:
+      MWAW_DEBUG_MSG(("MsWksSSParser::readCell: unexpected format\n"));
+      break;
+#if !defined(__clang__)
     default:
       f << ",subform==##unkn" << subformat;
       break;
+#endif
     }
-    break;
-  case MWAWCell::F_TEXT:
-    break;
-  case MWAWCell::F_BOOLEAN:
-  case MWAWCell::F_UNKNOWN:
-    MWAW_DEBUG_MSG(("MsWksSSParser::readCell: unexpected format\n"));
-    break;
-  default:
-    f << ",subform==##unkn" << subformat;
-    break;
   }
-
   long pos = debPos;
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
@@ -1043,7 +1063,7 @@ bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal
       if (content.m_textEntry.length()>=0) {
         std::string text("");
         for (long i=0; i <content.m_textEntry.length(); ++i)
-          text += (char) input->readULong(1);
+          text += char(input->readULong(1));
         f << text << ",";
       }
       else
@@ -1076,7 +1096,9 @@ bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal
     case MWAWCellContent::C_NONE:
       break;
     case MWAWCellContent::C_UNKNOWN:
+#if !defined(__clang__)
     default:
+#endif
       ok = false;
     }
     if (!ok)
@@ -1085,7 +1107,7 @@ bool MsWksSSParser::readCell(int sz, Vec2i const &cellPos, MsWksSSParserInternal
   cell.setFormat(format);
   // change the reference date from 1/1/1904 to 1/1/1900
   if (format.m_format==MWAWCell::F_DATE && content.isValueSet())
-    content.setValue(content.m_value+1462.);
+    content.setValue(content.m_value+1460.);
 
   std::string extra = f.str();
   f.str("");
@@ -1124,28 +1146,31 @@ bool MsWksSSParser::sendSpreadsheet()
   size_t numCell = sheet.m_cells.size();
 
   int prevRow = -1;
-  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols,76), librevenge::RVNG_POINT, sheet.m_name);
+  listener->openSheet(sheet.convertInPoint(sheet.m_widthCols,76), librevenge::RVNG_POINT, std::vector<int>(), sheet.m_name);
   // fixme: this sends the pictures present in the main page, but not the picture present in header/footer
   MsWksDocument::Zone zone=m_document->getZone(MsWksDocument::Z_MAIN);
   m_document->getGraphParser()->sendAll(zone.m_zoneId, true);
 
   for (size_t i = 0; i < numCell; i++) {
     MsWksSSParserInternal::Cell const &cell= sheet.m_cells[i];
-    // FIXME: openSheetRow/openSheetCell must do that
-    if (cell.position()[1] != prevRow) {
-      while (cell.position()[1] > prevRow) {
-        if (prevRow != -1)
-          listener->closeSheetRow();
-        prevRow++;
-        listener->openSheetRow(16, librevenge::RVNG_POINT);
-      }
+    // FIXME: find the row height...
+    if (cell.position()[1]>prevRow+1) {
+      if (prevRow != -1) listener->closeSheetRow();
+      int numRepeat=cell.position()[1]-1-prevRow;
+      listener->openSheetRow(16, librevenge::RVNG_POINT, numRepeat);
+      prevRow+=numRepeat;
+    }
+    if (cell.position()[1] > prevRow) {
+      if (prevRow != -1) listener->closeSheetRow();
+      ++prevRow;
+      listener->openSheetRow(16, librevenge::RVNG_POINT);
     }
     listener->openSheetCell(cell, cell.m_content);
     if (cell.m_content.m_textEntry.valid()) {
       listener->setFont(cell.isFontSet() ? cell.getFont() : sheet.m_font);
       input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
       while (!input->isEnd() && input->tell()<cell.m_content.m_textEntry.end()) {
-        unsigned char c=(unsigned char) input->readULong(1);
+        unsigned char c=static_cast<unsigned char>(input->readULong(1));
         if (c==0xd)
           listener->insertEOL();
         else
